﻿using FishFactoryContracts.BindingModels;
using FishFactoryContracts.SearchModels;
using FishFactoryContracts.StoragesContracts;
using FishFactoryContracts.ViewModels;
using FishFactoryFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FishFactoryFileImplement.Implements
{
    public class CannedStorage : ICannedStorage
    {
        private readonly DataFileSingleton source;
        public CannedStorage()
        {
            source = DataFileSingleton.GetInstance();
        }
        public List<CannedViewModel> GetFullList()
        {
            return source.Canneds.Select(x => x.GetViewModel).ToList();
        }
        public List<CannedViewModel> GetFilteredList(CannedSearchModel model)
        {
            if (string.IsNullOrEmpty(model.CannedName))
            {
                return new();
            }
            return source.Canneds
            .Where(x => x.CannedName.Contains(model.CannedName))
           .Select(x => x.GetViewModel)
           .ToList();
        }
        public CannedViewModel? GetElement(CannedSearchModel model)
        {
            if (string.IsNullOrEmpty(model.CannedName) && !model.Id.HasValue)
            {
                return null;
            }
            return source.Canneds
            .FirstOrDefault(x =>
           (!string.IsNullOrEmpty(model.CannedName) && x.CannedName ==
           model.CannedName) ||
            (model.Id.HasValue && x.Id == model.Id))
            ?.GetViewModel;
        }
        public CannedViewModel? Insert(CannedBindingModel model)
        {
            model.Id = source.Canneds.Count > 0 ? source.Canneds.Max(x =>
           x.Id) + 1 : 1;
            var newCanned = Canned.Create(model);
            if (newCanned == null)
            {
                return null;
            }
            source.Canneds.Add(newCanned);
            source.SaveCanneds();
            return newCanned.GetViewModel;
        }
        public CannedViewModel? Update(CannedBindingModel model)
        {
            var canned = source.Canneds.FirstOrDefault(x => x.Id ==
           model.Id);
            if (canned == null)
            {
                return null;
            }
            canned.Update(model);
            source.SaveCanneds();
            return canned.GetViewModel;
        }
        public CannedViewModel? Delete(CannedBindingModel model)
        {
            var element = source.Canneds.FirstOrDefault(x => x.Id ==
           model.Id);
            if (element != null)
            {
                source.Canneds.Remove(element);
                source.SaveCanneds();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
