﻿using FishFactoryContracts.BindingModels;
using FishFactoryContracts.ViewModels;
using FishFactoryDataModels;
using FishFactoryDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace FishFactoryFileImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }
        public string ShopName { get; private set; } = string.Empty;
        public string Address { get; private set; } = string.Empty;
        public DateTime DateOpening { get; private set; }
        public int MaxCountCanneds { get; private set; }

        private Dictionary<int, (ICannedModel, int)>? _shopCanned = null;
        public Dictionary<int, int> CountCanned { get; private set; } = new();

        public Dictionary<int, (ICannedModel, int)> ListCanneds {
            get
            {
                if (_shopCanned == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _shopCanned = CountCanned.ToDictionary(x => x.Key, y => ((source.Canneds.FirstOrDefault(z => z.Id == y.Key) as ICannedModel)!, y.Value));
                }
                return _shopCanned;
            }
        }
        public static Shop? Create(ShopBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Address = model.Address,
                MaxCountCanneds = model.MaxCountCanneds,
                DateOpening = model.DateOpening,
                CountCanned = model.ListCanneds.ToDictionary(x => x.Key, x => x.Value.Item2),
            };
        }
        public static Shop? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ShopName = element.Element("ShopName")!.Value,
                Address = element.Element("Address")!.Value,
                DateOpening = Convert.ToDateTime(element.Element("DateOpening")!.Value),
                MaxCountCanneds = Convert.ToInt32(element.Element("MaxCountCanneds")!.Value),
                CountCanned = element.Element("Canneds")!.Elements("Canned").ToDictionary(
                    x => Convert.ToInt32(x.Element("Key")?.Value),
                    x => Convert.ToInt32(x.Element("Value")?.Value))
            };
        }
        public void Update(ShopBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            ShopName = model.ShopName;
            Address = model.Address;
            DateOpening = model.DateOpening;
            MaxCountCanneds = model.MaxCountCanneds;
            CountCanned = model.ListCanneds.ToDictionary(x => x.Key, x => x.Value.Item2);
            _shopCanned = null;
        }
        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Address = Address,
            ListCanneds = ListCanneds,
            DateOpening = DateOpening,
            MaxCountCanneds = MaxCountCanneds,
        };
        public XElement GetXElement => new("Shop",
             new XAttribute("Id", Id),
             new XElement("ShopName", ShopName),
             new XElement("Address", Address),
             new XElement("DateOpening", DateOpening),
             new XElement("MaxCountCanneds", MaxCountCanneds),
             new XElement("Canneds", CountCanned
                 .Select(x => new XElement("Canned",
                     new XElement("Key", x.Key),
                     new XElement("Value", x.Value))
                 ).ToArray()));
    }
}
